/*
    A central source to make 'api' calls to firebase/cloud endpoints
    Usage:
    import api from $lib/api
    let data = await api.quickbooks.authorize()
*/
// import { getFunctions, httpsCallable } from 'firebase/functions'
// const functions = getFunctions()
import { notifications } from '../stores/notifications';
import { functions } from './firebase';
import { 
// dbUser as dbUserStore,
activeOrganization as activeOrganizationStore,
// currentUser as currentUserStore,
 } from '../stores/user';
import { supabaseClient as supabase } from './supabase';
import { httpsCallable } from 'firebase/functions';
let activeOrganization;
activeOrganizationStore.subscribe((value) => {
    activeOrganization = value?.id;
});
// api.js
export default {
    organization: {
        users: {
            add: ({ email, role, organization_id, created_by, }) => callFunction('addUserToOrg', {
                email,
                role,
                organization_id,
                created_by,
            }),
        },
    },
    plaid: {
        createLinkToken: (data) => callFunction('plaid-createLinkToken', data),
        createStripeAchAccount: (data) => callFunction('plaid-createStripeAchAccount', data),
    },
    quickbooks: {
        authorize: () => callFunction('quickbooks-authorize'),
        getIncome: async (data = {}) => {
            // https://developer.intuit.com/app/developer/qbo/docs/api/accounting/all-entities/profitandloss
            // start_date, end_date, date_macro
            let res = await callFunction('quickbooks-getIncome', data);
            return res;
        },
        refreshToken: () => callFunction('quickbooks-refreshToken'),
    },
    stripe: {
        createCheckoutSession: async (data) => {
            const session = await post('/api/stripe/checkout-session', data);
            return session;
        },
        createOrder: async (data) => {
            const order = await post('/api/stripe/order', data);
            return order;
        },
        createPaymentIntent: (data) => {
            return post('/api/stripe/payment-intent', data);
        },
        getCheckoutSession: async (id) => {
            const session = await get('/api/stripe/checkout-session?session_id=' + id);
            return session;
        },
        getPaymentIntent: async (id) => {
            return get('/api/stripe/payment-intent?payment_intent_id=' + id);
        },
        getPrices: () => callFunction('stripe-customers-getPrices'),
        getPaymentMethods: () => {
            return get('/api/stripe/payment-methods');
        },
        getSources: () => {
            return get('/api/stripe/sources');
        },
        getAccount: () => {
            return get('/api/stripe/account');
        },
        createPortalLink: (data) => {
            return post('/api/stripe/portal-link', data);
        },
        createAccountLink: (data) => {
            return post('/api/stripe/account-link', data);
        },
        updateCustomer: (data) => {
            return put('/api/stripe/customer', data);
        },
        createLoginLink: (data) => {
            return post('/api/stripe/login-link', data);
        },
    },
    inviteUser: (data) => {
        return post('/api/invite', data);
    },
    // stripe: {
    // 	createCheckoutSession: data => callFunction('stripe-customers-createCheckoutLink', data),
    // 	createPaymentIntent: data => callFunction('stripe-customers-createPaymentIntent', data),
    // 	getCheckoutSession: id => callFunction('stripe-customers-getCheckoutSession', { id }),
    // 	getPrices: () => callFunction('stripe-customers-getPrices'),
    // 	getPaymentMethods: () => callFunction('stripe-customers-getPaymentMethods'),
    // 	getSources: () => callFunction('stripe-customers-getSources'),
    // 	getAccount: () => callFunction('stripe-accounts-getAccount'),
    // 	createPortalLink: data => callFunction('stripe-customers-createPortalLink', data),
    // 	attachBankAccount: data => callFunction('stripe-customers-attachBankAccount', data),
    // 	verifySource: data => callFunction('stripe-customers-verifySource', data),
    // 	createAccountLink: data => callFunction('stripe-accounts-createAccountLink', data),
    // 	createLoginLink: data => callFunction('stripe-accounts-createLoginLink', data),
    // },
};
function callFunction(name, data) {
    return httpsCallable(functions, name)(data)
        .then((result) => result.data)
        .catch((e) => {
        notifications.error(e.message);
        console.error(e);
    });
}
async function callSupabaseFunction(functionName, body) {
    const { data, error } = await supabase.functions.invoke(functionName, {
        headers: {
            'my-custom-header': 'my-custom-header-value',
        },
        body: JSON.stringify(body),
    });
    // if (error instanceof FunctionsHttpError) {
    // 	console.log('Function returned an error', error.message)
    // } else if (error instanceof FunctionsRelayError) {
    // 	console.log('Relay error:', error.message)
    // } else if (error instanceof FunctionsFet) {
    // 	console.log('Fetch error:', error.message)
    // }
}
async function post(endpoint, data) {
    const res = await fetch(endpoint, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
            'active-organization': activeOrganization,
        },
        body: JSON.stringify(data),
    });
    return await res.json();
}
async function put(endpoint, data) {
    const res = await fetch(endpoint, {
        method: 'PUT',
        headers: {
            'Content-Type': 'application/json',
            'active-organization': activeOrganization,
        },
        body: JSON.stringify(data),
    });
    return await res.json();
}
async function get(endpoint) {
    const res = await fetch(endpoint, {
        headers: {
            'Content-Type': 'application/json',
            'active-organization': activeOrganization,
        },
    });
    return await res.json();
}
