import { notifications } from '../stores/notifications';
import { dbUser as dbUserStore, currentUser as currentUserStore, activeOrganization as activeOrganizationStore, } from '../stores/user';
// import { db } from './firebasev9'
import { goto } from '$app/navigation';
import api from './api';
import { auth, db, storage } from './firebase';
import firebase from 'firebase/compat/app';
// import KSUID from 'ksuid'
import * as ulid from 'ulid';
import { getDoc as fbGetDoc, setDoc as fbSetDoc, addDoc, collection, updateDoc as fbUpdateDoc, deleteDoc as fbDeleteDoc, getDocs, query, where, Timestamp, doc, orderBy, limit, writeBatch, startAfter, startAt, arrayUnion, arrayRemove, increment, deleteField as fbDeleteField, } from 'firebase/firestore';
import { deleteObject, getDownloadURL, ref, uploadBytes, } from 'firebase/storage';
import { createUserWithEmailAndPassword, GithubAuthProvider, GoogleAuthProvider, sendEmailVerification, sendPasswordResetEmail, signInWithEmailAndPassword, signInWithPopup, signOut, updateProfile, } from 'firebase/auth';
// console.log(ulid.ulid())
let currentUser;
let dbUser;
let activeOrganization;
let created_by, updated_by;
const VERIFY_EMAIL = false;
dbUserStore.subscribe((value) => {
    dbUser = value;
});
currentUserStore.subscribe((value) => {
    if (value) {
        currentUser = value;
        created_by = value.uid;
        updated_by = value.uid;
    }
});
activeOrganizationStore.subscribe((value) => {
    activeOrganization = value;
});
const prefixes = {
    webhooks: 'wh_',
    organizations: 'org_',
    notifications: 'not_',
};
function handleError(error) {
    if (error instanceof Error) {
        notifications.error(error.message);
    }
    console.error(error);
}
function generateDocId(coll) {
    // return (prefixes[coll] || '') + KSUID.randomSync().string
    return (prefixes[coll] || '') + ulid.ulid();
}
export async function createDoc(coll, data) {
    try {
        return addDoc(collection(db, coll), data);
    }
    catch (error) {
        handleError(error);
    }
}
export async function getDoc(path) {
    try {
        const docSnap = await fbGetDoc(doc(db, path));
        return docSnap.exists()
            ? { ...docSnap.data(), id: docSnap.id }
            : null;
    }
    catch (error) {
        handleError(error);
        return null;
        // return null as Type
    }
}
export async function getCollection(path) {
    try {
        let snap = await getDocs(query(collection(db, path)));
        if (snap.size) {
            return snap.docs.map((d) => ({
                ...d.data(),
                id: d.id,
            }));
        }
        else
            return [];
    }
    catch (error) {
        handleError(error);
        return [];
    }
}
// let things = await getCollection<User>('users')
// things.forEach((thing) => {
// 	thing.display_name
// })
export async function updateDoc(path, data, message) {
    try {
        const d = doc(db, path);
        await fbUpdateDoc(d, data);
        if (message)
            notifications.success(message);
    }
    catch (error) {
        handleError(error);
    }
}
export async function setDoc(path, data) {
    try {
        const d = doc(db, path);
        return fbSetDoc(d, data, { merge: true })
            .then(() => true)
            .catch((error) => {
            handleError(error);
            return false;
        });
    }
    catch (error) {
        handleError(error);
        return false;
    }
}
export async function deleteDoc(path, message) {
    try {
        const d = doc(db, path);
        return fbDeleteDoc(d).then(() => message && notifications.success(message));
    }
    catch (error) {
        handleError(error);
    }
}
export function uploadFile(file, path) {
    return uploadBytes(ref(storage, path), file)
        .then((snapshot) => {
        return getDownloadURL(snapshot.ref).then((url) => {
            return url;
        });
    })
        .catch((error) => handleError(error));
}
export function now() {
    return Timestamp.now();
}
export function deleteField() {
    return fbDeleteField();
}
export const Admin = {
    users: {
        getMany() {
            return getCollection('users');
        },
    },
};
// const things = await Admin.users.getMany()
// things.forEach(thing => {
// 	thing
// })
export const Auth = {
    resetPassword({ email, continueUrl, }) {
        return sendPasswordResetEmail(auth, email, { url: continueUrl })
            .then(() => {
            notifications.success('Reset email sent succesfully');
        })
            .catch((error) => {
            handleError(error);
        });
    },
    resendVerificationEmail() {
        if (auth.currentUser) {
            return sendEmailVerification(auth.currentUser)
                .then(() => {
                notifications.success('verification sent');
            })
                .catch((error) => handleError(error));
        }
    },
    resetEmail(email) {
        return sendPasswordResetEmail(auth, email)
            .then(() => {
            notifications.success('Reset link sent succesfully');
        })
            .catch((error) => {
            notifications.error(error.message);
        });
    },
    signIn({ email, password }) {
        return signInWithEmailAndPassword(auth, email, password)
            .then((userCredential) => {
            const user = userCredential.user;
            localStorage.setItem('uid', user.id);
            goto(`/`);
        })
            .catch((error) => {
            console.error(error.message);
            handleError(error);
        });
    },
    signInWithProvider(service) {
        let provider = null;
        if (service === 'github')
            provider = new GithubAuthProvider();
        if (service === 'google')
            provider = new GoogleAuthProvider();
        if (!provider)
            return;
        return signInWithPopup(auth, provider)
            .then(function (result) {
            const credential = GoogleAuthProvider.credentialFromResult(result);
            if (credential) {
                // const token = credential.accessToken
                // const user = result.user
                goto('/');
            }
        })
            .catch((error) => handleError(error));
    },
    signOut() {
        return fbSignOut(auth);
        // localStorage.removeItem('uid')
    },
    signUp({ email, password, continueUrl = window.location.origin, }) {
        return createUserWithEmailAndPassword(auth, email, password)
            .then(function (userCredential) {
            let { user } = userCredential;
            currentUser = user;
            updateProfile(user, { displayName: email });
            if (VERIFY_EMAIL) {
                sendEmailVerification(user, { url: continueUrl }).then(() => {
                    goto('/');
                });
            }
            goto('/');
        })
            .catch((error) => handleError(error));
    },
    update(data) {
        data.displayName = data.firstName + ' ' + data.lastName;
        updateProfile(auth.currentUser, { displayName: data.displayName });
        return updateDoc(`users/${currentUser.uid}`, data);
    },
};
export const Users = {
    // update(id, data) {
    // 	return setDoc(`users/${id}`, data)
    // },
    updateCurrent(data) {
        return setDoc(`users/${currentUser.uid}`, data);
    },
    update(id, data) {
        data.display_name = data.first_name + ' ' + data.last_name;
        updateProfile(auth.currentUser, { displayName: data.display_name });
        return setDoc(`users/${id}`, data).then(() => notifications.success('Updated'));
    },
    async get(id) {
        return getDoc(`users/${id}`);
    },
    addPhoto(file) {
        return uploadFile(file, `users/${currentUser.uid}`);
    },
};
export const Organizations = {
    add(data) {
        return createDoc('organizations', {
            ...data,
            created_by,
            owner: created_by,
        }).then(async (d) => {
            if (!d?.id)
                return;
            // await this.users.add({ ...dbUser, organization_id: doc.id, created_by })
            await setDoc(`user_organization/${created_by}-${d.id}`, {
                organization_id: d.id,
                user_id: created_by,
                accepted: true,
                role: 'owner',
                created_at: new Date(),
                last_active: new Date(),
            });
        });
    },
    get(id) {
        if (activeOrganization && activeOrganization.id) {
            return getDoc(`organizations/${activeOrganization.id}`);
        }
        else {
            return {};
        }
    },
    // TODO
    async getForUser() {
        const mapped = await getDocs(query(collection(db, 'user_organization'), where('user_id', '==', currentUser?.uid), orderBy('last_active', 'desc')))
            .then(async (snap) => {
            let orgs = await Promise.all(snap.docs.map(async (d) => {
                let data = d.data();
                let org = await fbGetDoc(doc(db, `organizations/${data.organization_id}`));
                return {
                    ...org.data(),
                    id: org.id,
                    role: data.role,
                    last_active: data.last_active,
                };
            }));
            // return orgs.sort((a, b) => (a.last_active > b.last_active ? -1 : 1))
            return orgs;
        })
            .catch((error) => handleError(error));
        return mapped || [];
    },
    update(data) {
        return updateDoc(`organizations/${activeOrganization.id}`, data);
    },
    addPhoto(file) {
        return uploadFile(file, `organizations/${activeOrganization.id}`);
    },
    users: {
        add: async ({ email, role }) => {
            const { data, error } = await api.organization.users.add({
                email,
                role,
                organization_id: activeOrganization.id,
                created_by,
            });
            if (error) {
                handleError(error);
                return error;
            }
            else {
                notifications.success('Success');
            }
            return data;
        },
        remove: async (user_id) => {
            return deleteDoc(`user_organization/${user_id}-${activeOrganization.id}`, 'User removed');
        },
        // TODO
        async get() {
            const mapped = await getDocs(query(collection(db, 'user_organization'), where('organization_id', '==', activeOrganization.id)))
                .then(async (snap) => {
                let users = await Promise.all(snap.docs.map(async (d) => {
                    let data = d.data();
                    let u = await fbGetDoc(doc(db, `users/${data.user_id}`));
                    return {
                        ...u.data(),
                        id: u.id,
                        role: data.role,
                        last_active: data.last_active,
                    };
                }));
                return users;
            })
                .catch((error) => handleError(error));
            return mapped || [];
        },
        async update(user_id, data) {
            return updateDoc(`user_organization/${user_id + '-' + activeOrganization.id}`, data);
        },
    },
};
export const Notifications = {
    get() {
        return getCollection(`users/${currentUser.uid}/webhooks`);
    },
    updateMany(notifications = [], data) {
        const batch = writeBatch(db);
        notifications.forEach((n) => {
            const notificationRef = doc(db, n.docPath);
            batch.update(notificationRef, data);
        });
        batch.commit();
    },
};
export const Webhooks = {
    add({ name = '', url, events = [] }) {
        return createDoc(`users/${currentUser.uid}/webhooks`, { name, url, events, status: 'active' }, 'Webhook added');
    },
    delete(id) {
        return deleteDoc(`users/${currentUser.uid}/webhooks/${id}`, 'Webhook removed successfully');
    },
    get() {
        return getCollection(`users/${currentUser.uid}/webhooks`);
    },
    update(id, data) {
        return updateDoc(`users/${currentUser.uid}/webhooks/${id}`, data, 'Webhook updated successfully');
    },
};
