import dayjs from 'dayjs';
import relativeTime from 'dayjs/plugin/relativeTime.js';
import config from '../config';
import { notifications } from '../stores/notifications';
import { Users } from './db';
import { db } from './firebase';
import { formatRelative } from 'date-fns';
export function slug(s) {
    const a = 'àáâäæãåāăąçćčđďèéêëēėęěğǵḧîïíīįìłḿñńǹňôöòóœøōõőṕŕřßśšşșťțûüùúūǘůűųẃẍÿýžźż·/_,:;';
    const b = 'aaaaaaaaaacccddeeeeeeeegghiiiiiilmnnnnoooooooooprrsssssttuuuuuuuuuwxyyzzz--';
    const p = new RegExp(a.split('').join('|'), 'g');
    return s
        .toString()
        .toLowerCase()
        .replace(/\s+/g, '-') // Replace spaces with -
        .replace(p, (/** @type {string} */ c) => b.charAt(a.indexOf(c))) // Replace special characters
        .replace(/&/g, '-and-') // Replace & with 'and'
        .replace(/[^\w\-]+/g, '') // Remove all non-word characters
        .replace(/\-\-+/g, '-') // Replace multiple - with single -
        .replace(/^-+/, '') // Trim - from start of text
        .replace(/-+$/, ''); // Trim - from end of text
}
export function copyToClipboard(text) {
    navigator.clipboard.writeText(text);
    notifications.success('Copied to clipboard');
}
// Returns a de-dupcliated array by a certain property
/**
 * @param {{ map: (arg0: (i: any) => any[]) => Iterable<readonly [any, any]>; }} array
 * @param {string | number} key
 */
export function uniqueByKey(array, key) {
    return [
        ...new Map(array.map((/** @type {{ [x: string]: any; }} */ i) => [i[key], i])).values(),
    ];
}
export function formatDateTime(dateString, onlyDate = false) {
    return onlyDate
        ? new Date(dateString).toLocaleDateString()
        : new Date(dateString).toDateString();
}
export function displayFirestoreTimestamp(firestoreTimeStamp, onlyDate = false) {
    if (firestoreTimeStamp && firestoreTimeStamp.toDate()) {
        return onlyDate
            ? firestoreTimeStamp.toDate().toDateString()
            : firestoreTimeStamp.toDate().toLocaleString();
    }
    else {
        return '';
    }
}
export function clickOutside(node) {
    const handleClick = (event) => {
        if (!node.contains(event.target)) {
            node.dispatchEvent(new CustomEvent('outclick'));
        }
    };
    document.addEventListener('click', handleClick, true);
    return {
        destroy() {
            document.removeEventListener('click', handleClick, true);
        },
    };
}
export function displayRelativeTime(ts) {
    dayjs.extend(relativeTime);
    const timestamp = ts && ts.toMillis ? ts.toMillis() : ts;
    return dayjs(timestamp).fromNow();
}
export function getFlag(countryCode) {
    try {
        return countryCode
            .toUpperCase()
            .replace(/./g, (/** @type {string} */ char) => String.fromCodePoint(char.charCodeAt(0) + 127397));
    }
    catch (error) {
        console.error(error);
    }
}
export function getCountryName(countryCode) {
    try {
        let country = config.countries.find((i) => i.code == countryCode);
        return country?.country || '';
    }
    catch (error) { }
}
export function getFileName(url) {
    const splited = url.split(/(%2F)..*(%2F)/);
    const fileName = splited[splited.length - 1].split('?')[0];
    return fileName;
}
export function processUrl(url) {
    return !/^https?:\/\//i.test(url) ? 'http://' + url : url;
}
export const preferences = {
    get: (dbUser, preference) => dbUser && dbUser.preferences && dbUser.preferences[preference],
    set: (currentUser, preference, value) => {
        try {
            Users.update(currentUser.uid, {
                preferences: {
                    [preference]: value,
                },
            });
        }
        catch { }
    },
};
export const auth = {
    isAdmin: (dbUser) => dbUser.is_admin,
};
