import { createClient } from '@supabase/supabase-js';
import { PUBLIC_SUPABASE_URL } from '$env/static/public';
import { SUPABASE_SERVICE_ROLE_KEY } from '$env/static/private';
import Stripe from 'stripe';
export const supabaseAdmin = createClient(PUBLIC_SUPABASE_URL || '', SUPABASE_SERVICE_ROLE_KEY || '');
export async function upsertTogglEntry(payload) {
    let res = await supabaseAdmin.from('toggl_entries').upsert({
        id: payload.id,
        at: payload.at,
        billable: payload.billable,
        start: payload.start,
        stop: payload.stop,
        workspace_id: payload.workspace_id,
        project_id: payload.project_id,
        user_id: payload.user_id,
        duration: payload.duration,
        duronly: payload.duronly,
    });
    return res;
}
export async function updateSubscription(subscription) {
    let res = await supabaseAdmin
        .from('organizations')
        .update({
        stripe_subscription_status: subscription.status,
        stripe_product: subscription.items.data[0].price.product,
    })
        .eq('stripe_customer_id', subscription.customer);
    return res;
}
export async function deleteRecord(table, id) {
    const { error } = await supabaseAdmin.from(table).delete().eq('id', id);
    if (error)
        console.error(error);
}
export async function upsertCustomer(customer) {
    const { error } = await supabaseAdmin.from('customers').upsert({
        id: customer.id,
        address: customer.address,
        email: customer.email,
        name: customer.name,
        description: customer.description,
        phone: customer.phone,
        metadata: customer.metadata,
        created: customer.created,
        default_source: customer.default_source,
        livemode: customer.livemode,
        currency: customer.currency,
        balance: customer.balance,
    });
    if (error)
        console.error(error);
}
export async function upsertInvoice(invoice) {
    const { error } = await supabaseAdmin.from('invoices').upsert({
        id: invoice.id,
        collection_method: invoice.collection_method,
        currency: invoice.currency,
        description: invoice.description,
        metadata: invoice.metadata,
        lines: invoice.lines,
        created: new Date(invoice.created * 1000),
        auto_advance: invoice.auto_advance,
        amount_remaining: invoice.amount_remaining,
        customer_email: invoice.customer_email,
        due_date: invoice.due_date,
        invoice_pdf: invoice.invoice_pdf,
        livemode: invoice.livemode,
        number: invoice.number,
        paid: invoice.paid,
        hosted_invoice_url: invoice.hosted_invoice_url,
        status: invoice.status,
        total: invoice.total,
        amount_due: invoice.amount_due,
        amount_paid: invoice.amount_paid,
        application_fee_amount: invoice.application_fee_amount,
        customer: invoice.customer,
    });
    if (error)
        console.error(error);
}
